import React, { useState } from 'react';
import { useHistory } from 'react-router-dom';
import { connect, useDispatch } from 'react-redux';

import { AppBar, Box, Toolbar, Typography, Menu, Container, Avatar, Tooltip, MenuItem, IconButton, Stack } from '@mui/material'
import { Instagram, AddAPhotoRounded, SearchRounded } from '@mui/icons-material';
import { actionFullLogout } from '../redux/action';












import { styled, alpha } from '@mui/material/styles';
import SearchIcon from '@mui/icons-material/Search';
import InputBase from '@mui/material/InputBase';










import { url } from '../../App';

function SearchField() {
    const Search = styled('div')(({ theme }) => ({
        position: 'relative',
        borderRadius: theme.shape.borderRadius,
        backgroundColor: alpha(theme.palette.common.white, 0.15),
        '&:hover': {
            backgroundColor: alpha(theme.palette.common.white, 0.25),
        },
        marginLeft: 0,
        width: '100%',
        [theme.breakpoints.up('sm')]: {
            marginLeft: theme.spacing(1),
            width: 'auto',
        },
    }));

    const SearchIconWrapper = styled('div')(({ theme }) => ({
        padding: theme.spacing(0, 2),
        height: '100%',
        position: 'absolute',
        pointerEvents: 'none',
        display: 'flex',
        alignItems: 'center',
        justifyContent: 'center',
    }));

    const StyledInputBase = styled(InputBase)(({ theme }) => ({
        color: 'inherit',
        '& .MuiInputBase-input': {
            padding: theme.spacing(1, 1, 1, 0),
            // vertical padding + font size from searchIcon
            paddingLeft: `calc(1em + ${theme.spacing(4)})`,
            transition: theme.transitions.create('width'),
            width: '100%',
            [theme.breakpoints.up('sm')]: {
                width: '12ch',
                '&:focus': {
                    width: '20ch',
                },
            },
        },
    }));



    return (
        <Search>
            <SearchIconWrapper>
                <SearchIcon />
            </SearchIconWrapper>
            <StyledInputBase
                placeholder="Search…"
                inputProps={{ 'aria-label': 'search' }}
            />
        </Search>
    )
}




function UserMenu({ login }) {
    let history = useHistory()
    const [anchorElUser, setAnchorElUser] = useState(null)
    const dispatch = useDispatch()


    const handleOpenUserMenu = (event) => {
        setAnchorElUser(event.currentTarget)
    }

    const handleCloseUserMenu = () => {
        setAnchorElUser(null)
    }

    function toMyAccount() {
        history.push(`/user/${login?._id}`)
    }

    function toCreatePost() {
        history.push('/createpost')
    }

    return (
        <Stack
            direction='row'
        >

            <SearchField />

            <IconButton
                sx={{
                    color: '#000'
                }}
                onClick={toCreatePost}
            >
                <AddAPhotoRounded />
            </IconButton>

            <Box sx={{ flexGrow: 0 }}>
                <Tooltip>
                    <IconButton
                        onClick={handleOpenUserMenu}
                    >
                        <Avatar
                            alt={login?.login}
                            src={url.slice(0, -7) + login?.avatar?.url}
                        />
                    </IconButton>
                </Tooltip>

                <Menu
                    sx={{
                        mt: '50px'
                    }}
                    id="menu-appbar"
                    anchorEl={anchorElUser}
                    anchorOrigin={{
                        vertical: 'top',
                        horizontal: 'right',
                    }}
                    keepMounted
                    transformOrigin={{
                        vertical: 'top',
                        horizontal: 'right',
                    }}
                    open={Boolean(anchorElUser)}
                    onClose={handleCloseUserMenu}
                >
                    <MenuItem
                        key={1}
                        onClick={toMyAccount}
                    >
                        <Typography
                            textAlign="center">
                            <span>
                                {login?.login}
                            </span>
                        </Typography>
                    </MenuItem>

                    <MenuItem
                        key={2}
                        onClick={() => { dispatch(actionFullLogout()) }}
                    >
                        <Typography
                            textAlign="center"
                        >
                            <span>
                                Выйти
                            </span>
                        </Typography>
                    </MenuItem>
                </Menu>
            </Box>
        </Stack>
    )
}

function Header({ login }) {
    let history = useHistory()

    function toMain() {
        history.push('/');
    }

    return (
        <AppBar
            position='sticky'
            color='inherit'
        >
            <Container
                maxWidth="xl"
            >
                <Toolbar
                    disableGutters
                >
                    <Stack
                        direction='row'
                        sx={{
                            display: 'flex',
                            flexGrow: '1'
                        }}
                    >
                        <IconButton
                            edge='start'
                            sx={{
                                color: '#000'
                            }}
                            onClick={toMain}
                        >
                            <Instagram />

                            <Typography
                                variant="h6"
                                sx={{
                                    fontFamily: 'monospace',
                                    fontWeight: 700,
                                    letterSpacing: '.3rem',
                                    ml: 1
                                }}
                            >
                                Hipstagram
                            </Typography>
                        </IconButton>
                    </Stack>

                    {localStorage.authToken && <UserMenu login={login} />}

                </Toolbar>
            </Container>
        </AppBar>
    )
}

export const CHeader = connect(state => ({ login: state?.promise?.AboutMe?.payload }))(Header)